/*
*
*      AggregPacking : Generator of random aggregate packings in concrete     
*                                                                             
*                           version 0.1 (2014-08-22)                          
* 
*                      Copyright (C)  2014  Jan Stransky                      
* 
*          Czech Technical University, Faculty of Civil Engineering,          
*      Department of Structural Mechanics, 166 29 Prague, Czech Republic      
* 
* AggregPacking is free software: you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as published by the
* Free Software Foundation, either version 3 of the License, or (at your
* option) any later version.
* 
* AggregPacking is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
* for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include "concmixture.hpp"

#ifdef AGGREGPACKING_BOOST_PYTHON
#include <boost/python.hpp>
#endif

namespace aggregpacking {


/**********************************************************************
* CONCRETE MIXTURE
**********************************************************************/
ConcreteMixture::ConcreteMixture() { // TODO?
	cementDensity = 3100.;
	aggregatesBulkDensity = 2590.;
	waterDensity = 1000.;
	ac = 4.5;
	wc = 0.45;
	update();
}

void
ConcreteMixture::update() {
	// Va + Vc + Vw = 1 m3 = ma/rhoa + mc/rhoc + mw/rhow = mc(ma/mc/rhoa + 1/rhoc + mw/mc/rhow) ( = 1 )
	double mc = 1. / ( 1./cementDensity + wc/waterDensity + ac/aggregatesBulkDensity);
	double ma = mc*ac;
	aggregateVolume = ma / aggregatesBulkDensity;
}

/**********************************************************************
* PYTHON
**********************************************************************/
#ifdef AGGREGPACKING_BOOST_PYTHON
/// @cond DOXYGEN_EXCLUDE
void py_concmixture() {
	using namespace boost::python;

	class_<ConcreteMixture,boost::noncopyable>("ConcreteMixture",init<>())
		;
}
/// @endcond
#endif

} // namespace
