/*
*
*      AggregPacking : Generator of random aggregate packings in concrete     
*                                                                             
*                           version 0.1 (2014-08-22)                          
* 
*                      Copyright (C)  2014  Jan Stransky                      
* 
*          Czech Technical University, Faculty of Civil Engineering,          
*      Department of Structural Mechanics, 166 29 Prague, Czech Republic      
* 
* AggregPacking is free software: you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as published by the
* Free Software Foundation, either version 3 of the License, or (at your
* option) any later version.
* 
* AggregPacking is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
* for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * @file
 * @brief File containing representation of concrete mixture
 */

#ifndef CONCRETEMIXTURE_HPP
#define CONCRETEMIXTURE_HPP

namespace aggregpacking {
/**********************************************************************
* CONCRETE MIXTURE
**********************************************************************/
/// Representation of concrete mixture
/**
 * Defined by all relevant physical properties
 */
class ConcreteMixture {
	public:
		/// Default constructor
		ConcreteMixture();

		/// Density of cement (default 3100)
		double cementDensity;

		/// Aggregate bulk density (default 2590)
		double aggregatesBulkDensity;

		/// Density of water (default 1000)
		double waterDensity;

		/// Aggregate/cement mass ratio (default 4.5)
		double ac;

		/// Water/cement mass ratio (default 0.45)
		double wc;

		/// Volume of aggregates in 1 m3 of concrete
		double aggregateVolume;

		/// Computes and sets aggrgate volume
		void update();
};


/**********************************************************************
* PYTHON
**********************************************************************/
#ifdef AGGREGPACKING_BOOST_PYTHON
/// @cond DOXYGEN_EXCLUDE
void py_concmixture();
/// @endcond
#endif

} // namespace
#endif
