import sys
import time
from aggregpacking import *

t = time.time()
cm = ConcreteMixture()
r  = FullerCurveRandomizer(10000,.5,1e-3,16e-3)
r.seed = 0
t = time.time()
hg = CubeHeteroGeometry(50e-3)
hg.aggregType = "Ellipsoid"
hg.placeNumTries = 1000000
hg.build(r,cm,0.01)
print 'Elapsed time: %g s (%d aggregates)'%(time.time()-t,len(hg.aggregates))

def unitOctahedron():
	nodes = [
		Vector3( 1, 0, 0),
		Vector3(-1, 0, 0),
		Vector3( 0, 1, 0),
		Vector3( 0,-1, 0),
		Vector3( 0, 0, 1),
		Vector3( 0, 0,-1),
	]
	elems = [
		( 0, 4, 2 ),
		( 2, 4, 1 ),
		( 1, 4, 3 ),
		( 3, 4, 0 ),
		( 0, 2, 5 ),
		( 2, 1, 5 ),
		( 1, 3, 5 ),
		( 3, 0, 5 )
	]
	return nodes,elems

def unitSphere(level=0):
	nodes,elems = unitOctahedron()
	for i in xrange(level):
		newElems = []
		centers = {}
		for elem in elems:
			i1,i2,i3 = [i for i in elem]
			n1,n2,n3 = [nodes[i] for i in elem]
			iExtra = 3*[None]
			for index,(i,j) in enumerate(((i1,i2),(i2,i3),(i3,i1))):
				bij,bji = centers.has_key((i,j)), centers.has_key((j,i))
				if bij or bji:
					iExtra[index] = centers[(i,j)] if bij else centers[(j,i)]
				else:
					ii = len(nodes)
					iExtra[index] = ii
					nn = (nodes[i]+nodes[j]).normalized()
					centers[(i,j)] = ii
					nodes.append(nn)
			i4,i5,i6 = iExtra
			newElems.extend((
				(i1,i4,i6),
				(i4,i2,i5),
				(i5,i3,i6),
				(i4,i5,i6),
			))
		elems = newElems
	return nodes,elems

def triangulateEllipsoid(e,level=0):
	nodes,elems = unitSphere(level)
	mm = e.matrixBTInversed
	#print mm
	#print mm.transposed()*mm
	return [e.pos+mm*n for n in nodes],elems

def save(base,level=0):
	nodes1,elems1 = [],[]
	nodes2,elems2 = [],[]
	sizes1,sizes2 = [],[]
	nTriesPerAggreg = None
	#ee = []
	for j,e in enumerate(hg.aggregates):
		nodes,elems,sizes = (nodes1,elems1,sizes1) if e.pos[0] > 25e-3 else (nodes2,elems2,sizes2)
		l = len(nodes)
		enodes,eelems = triangulateEllipsoid(e,level)
		nodes.extend(enodes)
		elems.extend([[i+l for i in elem] for elem in eelems])
		nTriesPerAggreg = len(eelems)
		sizes.append(e.size)
		#ee.append(e)
		#if j>=20: break
	for i,(nodes,elems,sizes) in enumerate(((nodes1,elems1,sizes1),(nodes2,elems2,sizes2))):
		f = open(base+'%d.vtk'%i,'w')
		f.write('# vtk DataFile Version 2.0\ncomment\nASCII\nDATASET POLYDATA\n')
		f.write('POINTS %d double\n'%len(nodes))
		f.writelines('%g %g %g\n'%(n[0],n[1],n[2]) for n in nodes)
		f.write('POLYGONS %d %d\n'%(len(elems),4*len(elems)))
		f.writelines('3 %d %d %d\n'%(e[0],e[1],e[2]) for e in elems)
		f.write('CELL_DATA %d\nSCALARS size double 1\nLOOKUP_TABLE default\n'%len(elems))
		for size in sizes:
			f.writelines('%g\n'%size for _ in xrange(nTriesPerAggreg))
		f.close()
		"""
	for i in xrange(len(ee)-1):
		#continue
		for j in xrange(i+1,len(ee)):
			ei = ee[i]
			ej = ee[j]
			ed,d = ei.computeEstimatedDistanceFrom(ej), ei.computeDistanceFrom(ej)
			if ed<=0 or d<=0:
				print i,j,ed,d
		"""

def save(base,level=0,lim=2e-3):
	nodes1,nodes2 = open('/tmp/nodes1','w'),open('/tmp/nodes2','w')
	elems1,elems2 = open('/tmp/elems1','w'),open('/tmp/elems2','w')
	sizes1,sizes2 = open('/tmp/sizes1','w'),open('/tmp/sizes2','w')
	nnodes1,nnodes2 = [0],[0]
	nelems1,nelems2 = [0],[0]
	#ee = []
	for j,ell in enumerate(hg.aggregates):
		nnodes,nelems,nodes,elems,sizes = (nnodes1,nelems1,nodes1,elems1,sizes1) if ell.pos[0] > 25e-3 else (nnodes2,nelems2,nodes2,elems2,sizes2)
		l = nnodes[0]
		ll = level if ell.size > lim else max(level-1,0)
		enodes,eelems = triangulateEllipsoid(ell,ll)
		nodes.writelines('%g %g %g\n'%(n[0],n[1],n[2]) for n in enodes)
		elems.writelines('3 %d %d %d\n'%(e[0]+l,e[1]+l,e[2]+l) for e in eelems)
		sizes.writelines('%g\n'%ell.size for _ in eelems)
		nnodes[0] += len(enodes)
		nelems[0] += len(eelems)
		#if j>=20: break
	for f in (nodes1,nodes2,elems1,elems2,sizes1,sizes2):
		f.close()
	nodes1,nodes2 = open('/tmp/nodes1'),open('/tmp/nodes2')
	elems1,elems2 = open('/tmp/elems1'),open('/tmp/elems2')
	sizes1,sizes2 = open('/tmp/sizes1'),open('/tmp/sizes2')
	for i,(nnodes,nelems,nodes,elems,sizes) in enumerate(((nnodes1,nelems1,nodes1,elems1,sizes1),(nnodes2,nelems2,nodes2,elems2,sizes2))):
		nnodes = nnodes[0]
		nelems = nelems[0]
		f = open(base+'%d.vtk'%i,'w')
		f.write('# vtk DataFile Version 2.0\ncomment\nASCII\nDATASET POLYDATA\n')
		f.write('POINTS %d double\n'%nnodes)
		f.writelines(line for line in nodes)
		f.write('POLYGONS %d %d\n'%(nelems,4*nelems))
		f.writelines(line for line in elems)
		f.write('CELL_DATA %d\nSCALARS size double 1\nLOOKUP_TABLE default\n'%nelems)
		f.writelines(line for line in sizes)
		f.close()
	for f in (nodes1,nodes2,elems1,elems2,sizes1,sizes2):
		f.close()

save('/tmp/aggregsell',level=3)
if 1:
	f = open('/tmp/ellaux.dat','w')
	f.writelines('%g %g %g %g\n'%(a.pos[0],a.pos[1],a.pos[2],.5*a.size) for a in hg.aggregates)
	f.close()
import postproutils
postproutils.createDatFile('ell',r,hg)
