/*
*
*      AggregPacking : Generator of random aggregate packings in concrete     
*                                                                             
*                           version 0.1 (2014-08-22)                          
* 
*                      Copyright (C)  2014  Jan Stransky                      
* 
*          Czech Technical University, Faculty of Civil Engineering,          
*      Department of Structural Mechanics, 166 29 Prague, Czech Republic      
* 
* AggregPacking is free software: you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as published by the
* Free Software Foundation, either version 3 of the License, or (at your
* option) any later version.
* 
* AggregPacking is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
* for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

/** @mainpage
 * AggregPacking - Generator of random aggregate packings in concrete
 *
 * Core written in C++, using Python interface
 *
 * @author <a href="http://mech.fsv.cvut.cz/~stransky">Jan Strásnký</a> jan.stransky@fsv.cvut.cz
 * @version 0.1
 * @date 2014-08-22
 * @copyright GNU Lesser General Public License
 */

/** @file
 * @brief Main file, combining everythin together.
 *
 * Shared library libaggregpacking.so is created from this file.
 */

#include "concmixture.hpp"
#include "math.hpp"
#include "aggregate.hpp"
#include "randomizer.hpp"
#include "heterogeometry.hpp"
#include "octree.hpp"
using octree::OctreeNode;

#ifdef AGGREGPACKING_BOOST_PYTHON
#include <boost/python.hpp>
#include <boost/python/suite/indexing/vector_indexing_suite.hpp>
#endif

/// Namespace aggregpacking
namespace aggregpacking {

/**********************************************************************
* PYTHON
**********************************************************************/
#ifdef AGGREGPACKING_BOOST_PYTHON
// http://code.activestate.com/lists/python-cplusplus-sig/15987/
/// Template class for C++ vector to/from Python list conversion
template <class Container>
class vector_ptr_indexing_suite : public boost::python::vector_indexing_suite<Container, true, vector_ptr_indexing_suite<Container>>
{
	/// @cond DOXYGEN_EXCLUDE
	public:
		typedef boost::python::iterator<Container, boost::python::return_value_policy<boost::python::reference_existing_object>> def_iterator;
		template <class Class>
		static void extension_def(Class & cl) {
			boost::python::vector_indexing_suite<Container, true,
			vector_ptr_indexing_suite<Container>>::extension_def(cl);
			cl.def("__iter__", def_iterator());
		}
	/// @endcond
};

/// Python module definition
BOOST_PYTHON_MODULE(libaggregpacking)
{
	using namespace boost::python;

	class_<vector<double>,boost::noncopyable>("list",no_init) \
		.def(vector_indexing_suite<vector<double>>() ) \
		;
	class_<vector<OctreeNode<Aggregate*>*>,boost::noncopyable>("list",no_init) \
		.def(vector_ptr_indexing_suite<vector<OctreeNode<Aggregate*>*>>() ) \
		;
	class_<vector<Aggregate*>,boost::noncopyable>("list",no_init) \
		.def(vector_ptr_indexing_suite<vector<Aggregate*>>() ) \
		;
	
	py_concmixture();
	py_aggregate();
	py_randomizer();
	py_heterogeometry();
	octree::py_octree<Aggregate*>();
}
#endif

} // namespace
