/*
*
*      AggregPacking : Generator of random aggregate packings in concrete     
*                                                                             
*                           version 0.1 (2014-08-22)                          
* 
*                      Copyright (C)  2014  Jan Stransky                      
* 
*          Czech Technical University, Faculty of Civil Engineering,          
*      Department of Structural Mechanics, 166 29 Prague, Czech Republic      
* 
* AggregPacking is free software: you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as published by the
* Free Software Foundation, either version 3 of the License, or (at your
* option) any later version.
* 
* AggregPacking is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
* for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include "math.hpp"

namespace aggregpacking {

#ifdef AGGREGPACKING_SERIALIZATION
ostream&
operator<<(ostream &os, const Vector3& v) {
	os <<"Vector3 ( " << v.x() << " , " << v.y() << " , " << v.z() << " )";
	return os;
}

ostream&
operator<<(ostream &os, const Matrix3& m) {
	os << "Matrix3 ( " << m(0,0) << " , " << m(0,1) << " , " << m(0,2) << " , " << m(1,0) << " , " << m(1,1) << " , " << m(1,2) << " , " << m(2,0) << " , " << m(2,1) << " , " << m(2,2) << " )";
	return os;
}

ostream&
operator<<(ostream &os, const Quaternion& q) {
	AngleAxis aa(q);
	Vector3 axis = aa.axis();
	os <<"Quaternion (( " << axis.x() << " , " << axis.y() << " , " << axis.z() << " ), " << aa.angle() << " )";
	return os;
}

istream&
operator>>(istream& is, Vector3& v) {
	string str;
	double x,y,z;
	is >> str;
	is >> str;
	is >> v.x();
	is >> str;
	is >> v.y();
	is >> str;
	is >> v.z();
	is >> str;
	return is;
}

istream&
operator>>(istream& is, Matrix3& m) {
	string str;
	is >> str;
	is >> str;
	for (int i=0; i<3; i++) {
		for (int j=0; j<3; j++) {
			is >> m(i,j);
			is >> str;
		}
	}
	return is;
}

istream&
operator>>(istream& is, Quaternion& q) {
	string str;
	Vector3 axis;
	double a;
	is >> str;
	is >> str;
	is >> axis.x();
	is >> str;
	is >> axis.y();
	is >> str;
	is >> axis.z();
	is >> str;
	is >> a;
	is >> str;
	q = Quaternion(AngleAxis(a,axis));
	return is;
}
#endif
} // namespace
