/*
*
*      AggregPacking : Generator of random aggregate packings in concrete     
*                                                                             
*                           version 0.1 (2014-08-22)                          
* 
*                      Copyright (C)  2014  Jan Stransky                      
* 
*          Czech Technical University, Faculty of Civil Engineering,          
*      Department of Structural Mechanics, 166 29 Prague, Czech Republic      
* 
* AggregPacking is free software: you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as published by the
* Free Software Foundation, either version 3 of the License, or (at your
* option) any later version.
* 
* AggregPacking is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
* for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * @file
 * @brief File containing basic math declarations
 */

#ifndef MATH_HPP
#define MATH_HPP

#include <Eigen/Core>
#include <Eigen/Geometry>

#include <string>
using std::string;
#ifdef AGGREGPACKING_SERIALIZATION
#include <iostream>
#include <fstream>
using std::istream;
using std::ostream;
using std::ifstream;
using std::ofstream;
#endif

namespace aggregpacking {

/// PI
const double PI = 3.141592653589793238463;

/// 3 component vector of real numbers.
/** @class aggregpacking::Vector3
 * See math.hpp file or <a href="http://eigen.tuxfamily.org">Eigen documentation</a>
 */
typedef Eigen::Matrix<double,3,1> Vector3;

/// 3x3 matrix of real numbers
/** @class aggregpacking::Matrix3
 * See math.hpp file or Eigen documentation <http://eigen.tuxfamily.org>
 */
typedef Eigen::Matrix<double,3,3> Matrix3;

/// Quaternion - for rotation representation
/** @class aggregpacking::Quaternion
 * See math.hpp file or Eigen documentation <http://eigen.tuxfamily.org>
 */
typedef Eigen::Quaternion<double> Quaternion;

/// Another rotation representation
/** @class aggregpacking::AngleAxis
 * See math.hpp file or Eigen documentation <http://eigen.tuxfamily.org>
 */
typedef Eigen::AngleAxis<double> AngleAxis;


#ifdef AGGREGPACKING_SERIALIZATION
/// Vector3 to ostream shift operator
ostream& operator<<(ostream &os, const Vector3& v);

/// Matrix3 to ostream shift operator
ostream& operator<<(ostream &os, const Matrix3& m);

/// Quaternion to ostream shift operator
ostream& operator<<(ostream &os, const Quaternion& q);

/// Vector3 from istream shift operator
istream& operator>>(istream& is, Vector3& v);

/// Matrix3 from istream shift operator
istream& operator>>(istream& is, Matrix3& m);

/// Quaternion from istream shift operator
istream& operator>>(istream& is, Quaternion& q);

#endif

} // namespace
#endif
