#include <boost/python.hpp>

#include "../math.hpp"
#include "../randomizer.hpp"

namespace aggregpacking {

// VECTOR 3
/*
struct custom_Vector3_from_seq{
	custom_Vector3_from_seq(){
		 boost::python::converter::registry::push_back(&convertible,&construct,boost::python::type_id<Vector3>());
	}
	static void* convertible(PyObject* obj_ptr){
		 if(!PySequence_Check(obj_ptr)) return 0;
		 if(PySequence_Size(obj_ptr)!=3) return 0;
		 return obj_ptr;
	}
	static void construct(PyObject* obj_ptr, boost::python::converter::rvalue_from_python_stage1_data* data){
		void* storage=((boost::python::converter::rvalue_from_python_storage<Vector3>*)(data))->storage.bytes;
		new (storage) Vector3; Vector3* vec=(Vector3*)storage;
		for (int i=0; i<3; i++) (*vec)[i]=boost::python::extract<double>(PySequence_GetItem(obj_ptr,i));
		data->convertible=storage;
	}
};
*/

// SIEVE_CURVE
struct custom_sieve_curve_to_tuple{
	static PyObject* convert(const ExperimentalSieveCurveRandomizer::sieve_curve& sc){
		boost::python::list l;
		for (int i=0; i<sc.size(); i++) l.append(boost::python::make_tuple(sc[i][0],sc[i][1]));
		boost::python::tuple ret(l);
		return boost::python::incref(ret.ptr());
	}
};
struct custom_sieve_curve_from_seq{
	custom_sieve_curve_from_seq(){
		 boost::python::converter::registry::push_back(&convertible,&construct,boost::python::type_id<ExperimentalSieveCurveRandomizer::sieve_curve>());
	}
	static void* convertible(PyObject* obj_ptr){
		 if(!PySequence_Check(obj_ptr)) return 0;
		 PyObject* item;
		 for (int i=0; i<PySequence_Size(obj_ptr); i++) {
			 item = PySequence_GetItem(obj_ptr,i);
			 if (!PySequence_Check(item) || PySequence_Size(item)!=2) return 0;
		 }
		 return obj_ptr;
	}
	static void construct(PyObject* obj_ptr, boost::python::converter::rvalue_from_python_stage1_data* data){
		void* storage=((boost::python::converter::rvalue_from_python_storage<ExperimentalSieveCurveRandomizer::sieve_curve>*)(data))->storage.bytes;
		new (storage) ExperimentalSieveCurveRandomizer::sieve_curve; ExperimentalSieveCurveRandomizer::sieve_curve* sc=(ExperimentalSieveCurveRandomizer::sieve_curve*)storage;
		const int size = PySequence_Size(obj_ptr);
		sc->resize(size);
		for (int i=0; i<size; i++) {
			for (int j=0; j<2; j++) (*sc)[i][j] = boost::python::extract<double>(PySequence_GetItem(PySequence_GetItem(obj_ptr,i),j));
		}
		data->convertible=storage;
	}
};



BOOST_PYTHON_MODULE(_customConverters){
	//custom_Vector3_from_seq();

	custom_sieve_curve_from_seq(); 
	boost::python::to_python_converter<ExperimentalSieveCurveRandomizer::sieve_curve,custom_sieve_curve_to_tuple>();
}

}
