/*
*
*      AggregPacking : Generator of random aggregate packings in concrete     
*                                                                             
*                           version 0.1 (2014-08-22)                          
* 
*                      Copyright (C)  2014  Jan Stransky                      
* 
*          Czech Technical University, Faculty of Civil Engineering,          
*      Department of Structural Mechanics, 166 29 Prague, Czech Republic      
* 
* AggregPacking is free software: you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as published by the
* Free Software Foundation, either version 3 of the License, or (at your
* option) any later version.
* 
* AggregPacking is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
* for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * @file
 * @brief File containing several auxiliary functions
 */

#ifndef UTILS_HPP
#define UTILS_HPP

#include <stdexcept>

#include <iostream>
#ifdef AGGREGPACKING_SERIALIZATION
#include <fstream>
#include <string>
using std::istream;
using std::ostream;
using std::ifstream;
using std::ofstream;
using std::string;
#endif

namespace aggregpacking {

/**********************************************************************
* LOG
**********************************************************************/
#ifdef AGGREGPACKING_DEBUG
/// Debug log taking 1 argument
#define LOG_DEBUG1(msg1) std::cout<<__FILE__<<": "<<__LINE__<<": "<<msg1<<"\n";

/// Debug log taking 2 argument
#define LOG_DEBUG2(msg1,msg2) std::cout<<__FILE__<<": "<<__LINE__<<": "<<msg1<<" "<<msg2<<"\n";

/// Debug log taking 3 argument
#define LOG_DEBUG3(msg1,msg2,msg3) std::cout<<__FILE__<<": "<<__LINE__<<": "<<msg1<<" "<<msg2<<" "<<msg3<<"\n";

/// Debug log taking 4 argument
#define LOG_DEBUG4(msg1,msg2,msg3,msg4) std::cout<<__FILE__<<": "<<__LINE__<<": "<<msg1<<" "<<msg2<<" "<<msg3<<" "<<msg4<<"\n";

/// Debug log taking 5 argument
#define LOG_DEBUG5(msg1,msg2,msg3,msg4,msg5) std::cout<<__FILE__<<": "<<__LINE__<<": "<<msg1<<" "<<msg2<<" "<<msg3<<" "<<msg4<<" "<<msg5<<"\n";

/// Debug log taking 1 argument
#define LOG_DEBUG(msg) LOG_DEBUG1(msg)

#else
/// @cond DOXYGEN_EXCLUDE
#define LOG_DEBUG1(msg1)
#define LOG_DEBUG2(msg1,msg2)
#define LOG_DEBUG3(msg1,msg2,msg3)
#define LOG_DEBUG4(msg1,msg2,msg3,msg4)
#define LOG_DEBUG5(msg1,msg2,msg3,msg4,msg5)
#define LOG_DEBUG(msg)
/// @endcond
#endif

/// @cond DOXYGEN_EXCLUDE
#define __S1(x) #x
#define __S2(x) __S1(x)
#define __AGGREGPACKING_CERR std::cerr<<__FILE__<<": "<<__LINE__<<": "
#define __AGGREGPACKING_THROW throw std::runtime_error(std::string(__FILE__)+": "+__S2(__LINE__))
/// @endcond

/// Error log taking 1 argument
#define LOG_ERROR1(msg1) __AGGREGPACKING_CERR<<msg1<<"\n"; __AGGREGPACKING_THROW;

/// Error log taking 2 argument
#define LOG_ERROR2(msg1,msg2) __AGGREGPACKING_CERR<<msg1<<" "<<msg2<<"\n"; __AGGREGPACKING_THROW;

/// Error log taking 3 argument
#define LOG_ERROR3(msg1,msg2,msg3) __AGGREGPACKING_CERR<<msg1<<" "<<msg2<<" "<<msg3<<"\n"; __AGGREGPACKING_THROW;

/// Error log taking 4 argument
#define LOG_ERROR4(msg1,msg2,msg3,msg4) __AGGREGPACKING_CERR<<msg1<<" "<<msg2<<" "<<msg3<<" "<<msg4<<"\n"; __AGGREGPACKING_THROW;

/// Error log taking 5 argument
#define LOG_ERROR5(msg1,msg2,msg3,msg4,msg5) __AGGREGPACKING_CERR<<msg1<<" "<<msg2<<" "<<msg3<<" "<<msg4<<" "<<msg5<<"\n"; __AGGREGPACKING_THROW;

/// Error log taking 1 argument
#define LOG_ERROR(msg) LOG_ERROR1(msg)

/// Warn log taking 1 argument
#define LOG_WARN1(msg1) __AGGREGPACKING_CERR<<msg1<<"\n";

/// Warn log taking 2 argument
#define LOG_WARN2(msg1,msg2) __AGGREGPACKING_CERR<<msg1<<" "<<msg2<<"\n";

/// Warn log taking 3 argument
#define LOG_WARN3(msg1,msg2,msg3) __AGGREGPACKING_CERR<<msg1<<" "<<msg2<<" "<<msg3<<"\n";

/// Warn log taking 4 argument
#define LOG_WARN4(msg1,msg2,msg3,msg4) __AGGREGPACKING_CERR<<msg1<<" "<<msg2<<" "<<msg3<<" "<<msg4<<"\n";

/// Warn log taking 5 argument
#define LOG_WARN5(msg1,msg2,msg3,msg4,msg5) __AGGREGPACKING_CERR<<msg1<<" "<<msg2<<" "<<msg3<<" "<<msg4<<" "<<msg5<<"\n";

/// Warn log taking 1 argument
#define LOG_WARN(msg) LOG_WARN1(msg)


/**********************************************************************
* FILES
**********************************************************************/
#ifdef AGGREGPACKING_SERIALIZATION
/// Init file for ostream
/** @return true is succesful, false otherwise
 */
bool initFile(ofstream& f, const string& fName, const string& memoizeDb);

/// Init file for istream
/** @return true is succesful, false otherwise
 */
bool initFile(ifstream& f, const string& fName, const string& memoizeDb);

/// Auxiliary template function for saving objects to files
template <typename T>
void saveObject(const T& t, const string& fName, const string& memoizeDb) {
	ofstream f;
	if (!initFile(f,fName,memoizeDb)) return;
	f << t;
	f.close();
}

/// Auxiliary template function for lading objects from files
template <typename T>
bool loadObject(T& t, const string& fName, const string& memoizeDb) {
	ifstream f;
	if (!initFile(f,fName,memoizeDb)) return false;
	string s;
	f >> s;
	if (s != t.getClassName()) { LOG_ERROR2(t.getClassName(),"wrong indentification class name"); }
	f >> t; 
	f.close();
	return true;
}
#endif

} // namespace
#endif
